/************************************************************************************
****    Procedure para registro do perodo para apurao do resultado operacional ***
*************************************************************************************

Criado em  : 08/2007
       Por : Alessandra Cristina

Descrio:

           - Altera perodo programado para apurao do Resultado Operacional da Filial
             de acordo com a nova data de processamento parametrizada.

           - Caso a data j tenha sido apurada deleta registro da tabela ROPFILIAL, tabela
             utilizada para restringir reclculo do ROP.
Util :
       - Dir  : @C:\WEB_411\SQL\END_CONV\ORACLE\0018_GERTRGO_Atualiza_Dados_DtProcessa.sql

       - Erro : raise_application_error(-20501, 'Mensagem', True);

*/

CREATE OR REPLACE TRIGGER ATUALIZA_DADOS_DTPROCESSA
BEFORE UPDATE
ON PARAMFILIAL FOR EACH ROW
DECLARE
   V_DTINICIAL       DATE;
   V_DTFINAL         DATE;
   V_IDFECHDIAROP    CHAR;
   V_IDUTPADRROPFIL  CHAR;
   V_IDUTPADRROP     CHAR;

BEGIN

   IF UPDATING THEN
      SELECT MAX(IDUTPADRROP)
      INTO V_IDUTPADRROP
      FROM PARAMGERAL G;

      BEGIN
         SELECT IDFECHDIAROP, IDUTPADRROPFIL
         INTO V_IDFECHDIAROP, V_IDUTPADRROPFIL
         FROM  PARFILIALROP
         WHERE CDFILIAL = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN BEGIN
            V_IDFECHDIAROP   := NULL;
            V_IDUTPADRROPFIL := NULL;
         END;
      END;

      IF V_IDUTPADRROP = 'S' OR V_IDUTPADRROPFIL = 'S' THEN 
         BEGIN
            SELECT IDFECHDIAROP, IDUTPADRROPFIL
            INTO V_IDFECHDIAROP, V_IDUTPADRROPFIL
            FROM  PARFILIALROP
            WHERE CDFILIAL = 'P';
         EXCEPTION
            WHEN NO_DATA_FOUND THEN BEGIN
               V_IDFECHDIAROP := NULL;
               V_IDUTPADRROPFIL := NULL;
            END;
         END;
      END IF;

      IF (:NEW.DTPROCESSA <> :OLD.DTPROCESSA) THEN
         SELECT MAX(DTINIAPURACROP), MAX(DTFINAPURACROP)
         INTO V_DTINICIAL, V_DTFINAL
         FROM APURACAOROP
         WHERE CDFILIAL = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL);

         IF (NVL(V_IDFECHDIAROP,'N') <> 'S') THEN
            IF (:NEW.DTPROCESSA > V_DTFINAL) THEN
               UPDATE APURACAOROP
               SET DTFINAPURACROP  = :NEW.DTPROCESSA
               WHERE CDFILIAL = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL);

               DELETE ROPFILIAL
               WHERE CDFILIAL     = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL)  AND
                     DTFILIALROP  >= :NEW.DTPROCESSA;
            END IF;

            IF (:NEW.DTPROCESSA < V_DTFINAL)  AND (:NEW.DTPROCESSA < V_DTINICIAL) THEN
               UPDATE APURACAOROP
               SET DTINIAPURACROP  = :NEW.DTPROCESSA
               WHERE CDFILIAL = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL);
            END IF;
         ELSE
            DELETE ROPFILIAL
            WHERE CDFILIAL     = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL)  AND
                  DTFILIALROP  >= :NEW.DTPROCESSA;
            IF (:NEW.DTPROCESSA < V_DTFINAL)  THEN
               UPDATE APURACAOROP
               SET  DTFINAPURACROP  = :NEW.DTPROCESSA
               WHERE CDFILIAL = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL);
            END IF;
            IF (:NEW.DTPROCESSA < V_DTINICIAL) THEN
               UPDATE APURACAOROP
               SET  DTINIAPURACROP  = :NEW.DTPROCESSA
               WHERE CDFILIAL = NVL(:NEW.CDFILIAL, :OLD.CDFILIAL);
            END IF;
         END IF;
      END IF;
   END IF;
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('ATUALIZA_DADOS_DTPROCESSA','5.01.011'); END;
/

